# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import datetime
import logging

import six
from six import BytesIO, StringIO

from driverng import MessageV2, Worklist

logger = logging.getLogger("driverng.DxFlex")


class DbFlexResult(object):
    pass


class DbFlexResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: DbFlexResult}

    def __init__(self, separateur, resu_items):
        self.separateur = separateur
        self.resu_items = resu_items
        return super(DbFlexResultsMultiple, self).__init__()

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        if six.PY2:
            buf = BytesIO(data.encode("utf-8"))
        else:
            buf = StringIO(data)
        self.fields = []
        reader = csv.reader(buf, delimiter=str(self.separateur))
        self.date_resu = None
        self.instrument_serial_number = None
        obj = None
        for row in reader:
            if not row[0]:
                continue
            if row[0] == "Sample":
                obj = self.add(type_message=MessageV2.RESULT)
                continue
            if len(row) > 1:
                field = row[0].strip().replace(" ", "")
                self.fields.append(field)
                if field in self.resu_items:
                    setattr(obj, field, (row[1], row[2] if len(row) > 2 else ""))
                else:
                    setattr(obj, field, row[1])


class Driver(MessageV2):

    ENCODING = "utf-8"
    SPEC = {"code": "dxflex", "label": "Db Flex  "}

    SAMPLE = "SampleID"
    COMMENT = "ReviewedComment"
    ANALYSE = "Task"
    RESUS = ["NbrSpermatozoides", "Normaux", "FRAGMENTATION", "DECONDENSATION"]
    OPERATOR = "Testedby"
    DATE = "TestTime"
    OPTIONS = [
        MessageV2.Option(
            "separateur", label="Séparateur de champs du fichier", default=","
        )
    ]

    def __init__(self, **kwargs):
        self.separateur = kwargs.pop("separateur", ",")
        return super(Driver, self).__init__(**kwargs)

    def parse(self, data):
        obj = DbFlexResultsMultiple(self.separateur, self.RESUS)
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for m in message:
            sample_id = getattr(m, self.SAMPLE, None)
            if not sample_id:
                continue
            controle = sample_id.startswith("CQ")
            analyse = getattr(m, self.ANALYSE, "")
            operator = getattr(m, self.OPERATOR, "")
            comment = getattr(m, self.COMMENT, None)
            try:
                date = datetime.datetime.strptime(
                    getattr(m, self.DATE), "%Y-%m-%d %H:%M:%S"
                )
            except ValueError:
                date = None

            sample = worklist.get_or_create(
                sample_id=sample_id, type="control" if controle else "patient"
            )
            if controle:
                sample.control_lot = sample_id

            for attr in self.RESUS:
                resu = getattr(m, attr)[0].replace(",", ".").strip()
                if resu:
                    resultat = sample.results.add(
                        code="{}_{}".format(analyse, attr),
                        value=resu,
                        unit=getattr(m, attr)[1],
                        date=date,
                        operator=operator,
                        comments=[comment] if comment else [],
                    )
                if controle:
                    resultat.date = datetime.datetime.now().replace(
                        second=0, microsecond=0
                    )

        return worklist
